/*
 * Copyright (C) 2009 - 2010 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

/* $Id$ */

#include <limits.h>
#include "daemon/ProfileComponentsHolder.h"
#include "treemanager/IMOTreeManager.h"
#include "serverexchange/IServerExchangeManager.h"
#include "serverexchange/commands/ServerPollingInfoCommand.h"
#include "Utils.h"


using namespace NS_DM_Client;
using namespace NS_DM_Client::NS_Communication;


ServerPollingInfoCommand::ServerPollingInfoCommand(ProfileComponentsHolder *pHolder,
												   String &uriPSupported,
												   String &uriPInterval,
												   String &uriPAttempts) :
	m_pProfileHolder(pHolder),
	m_URIPSupported(uriPSupported),
	m_URIPInterval(uriPInterval),
	m_URIPAttempts(uriPAttempts)
{
}


ServerPollingInfoCommand::~ServerPollingInfoCommand()
{
}


bool ServerPollingInfoCommand::Execute()
{
	String result;

	bool pollingSupported = false;
	int  pollingInterval = 0; // seconds
	int  pollingAttempts = 0;

	if (e_Ok == m_pProfileHolder->GetMOTreeManager()->GetValue(m_URIPSupported, result, ""))
	{
		pollingSupported = IsBooleanTrue(result);
	}

	if (e_Ok == m_pProfileHolder->GetMOTreeManager()->GetValue(m_URIPInterval , result, ""))
	{
		pollingInterval = atoi(result.c_str());
		if (pollingInterval == INT_MAX || pollingInterval == INT_MIN)
		{
			pollingInterval = 0;
		}
	}

	if (e_Ok == m_pProfileHolder->GetMOTreeManager()->GetValue(m_URIPAttempts, result, ""))
	{
		pollingAttempts = atoi(result.c_str());
		if (pollingAttempts == INT_MAX || pollingAttempts == INT_MIN)
		{
			pollingAttempts = 0;
		}
	}

	m_pProfileHolder->GetServerExchangeManager()->SetServerPollerInfo(pollingSupported, pollingInterval, pollingAttempts);

    return true;
}

